@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2008-2009 Jens Vagelpohl and Contributors. All Rights Reserved.
+# Copyright (c) 2008-2012 Jens Vagelpohl and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
@@ -11,11 +11,8 @@
 #
 ##############################################################################
 """ unit tests base classes
-
-$Id: base.py 1901 2010-02-07 19:01:08Z jens $
 """
 
-import base64
 import unittest
 
 from dataflake.ldapconnection.connection import connection_cache
@@ -77,8 +74,7 @@ class LDAPConnectionTests(unittest.TestCase):
         record = fakeldap.addTreeItems(dn)
         for key, value in kw.items():
             if key.lower() == 'userpassword':
-                sha_digest = fakeldap.sha_new(value).digest()
-                value = ['{SHA}%s' % base64.encodestring(sha_digest).strip()]
+                value = [fakeldap.hash_pwd(value)]
             elif isinstance(value, basestring):
                 value = [value]
             record[key] = value
@@ -89,6 +85,7 @@ class FakeLDAPTests(unittest.TestCase):
     def setUp(self):
         from dataflake.ldapconnection.tests import fakeldap
         fakeldap.addTreeItems('ou=users,dc=localhost')
+        fakeldap.addTreeItems('ou=groups,dc=localhost')
 
     def tearDown(self):
         from dataflake.ldapconnection.tests import fakeldap
@@ -102,17 +99,37 @@ class FakeLDAPTests(unittest.TestCase):
         conn = self._getTargetClass()(*args, **kw)
         return conn
 
-
-    def _addUser(self, name):
+    def _addUser(self, name, mail=None):
         conn = self._makeOne()
         user_dn = 'cn=%s,ou=users,dc=localhost' % name
         user_pwd = '%s_secret' % name
-        sha_digest = fakeldap.sha_new(user_pwd).digest()
-        pwd = '{SHA}%s' % base64.encodestring(sha_digest).strip()
+
+        if conn.hash_password:
+            pwd = fakeldap.hash_pwd(user_pwd)
+        else:
+            pwd = user_pwd
+
         user = [ ('cn', [name])
                , ('userPassword', [pwd])
                , ('objectClass', ['top', 'person'])
                ]
+        if mail is not None:
+            user.append(('mail', [mail]))
+
         conn.add_s(user_dn, user)
         return (user_dn, user_pwd)
 
+    def _addGroup(self, name, members=None):
+        conn = self._makeOne()
+        group_dn = 'cn=%s,ou=groups,dc=localhost' % name
+
+        group = [ ('cn', [name])
+                , ('objectClass', ['top', 'group'])
+                ]
+        if members is not None:
+            members = ['cn=%s,ou=users,dc=localhost' % x for x in members]
+            group.append((conn.member_attr, members))
+
+        conn.add_s(group_dn, group)
+        return group_dn
+
