@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2008-2010 Jens Vagelpohl and Contributors. All Rights Reserved.
+# Copyright (c) 2008-2012 Jens Vagelpohl and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
@@ -11,8 +11,6 @@
 #
 ##############################################################################
 """ test_connection_delete: Tests for the LDAPConnection delete method
-
-$Id: test_connection_delete.py 1901 2010-02-07 19:01:08Z jens $
 """
 
 import unittest
@@ -29,7 +27,8 @@ class ConnectionDeleteTests(LDAPConnectionTests):
         conn.delete('cn=foo,dc=localhost')
         connection = conn._getConnection()
         binduid, bindpwd = connection._last_bind[1]
-        self.assertEqual(binduid, u'')
+        self.assertTrue(isinstance(binduid, str))
+        self.assertEqual(binduid, '')
         self.assertEqual(bindpwd, '')
 
     def test_delete_authentication(self):
