@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2008-2010 Jens Vagelpohl and Contributors. All Rights Reserved.
+# Copyright (c) 2008-2012 Jens Vagelpohl and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
@@ -11,8 +11,6 @@
 #
 ##############################################################################
 """ test_connection_modify: Tests for the LDAPConnection modify method
-
-$Id: test_connection_modify.py 1916 2010-04-12 11:50:39Z jens $
 """
 
 import unittest
@@ -34,7 +32,8 @@ class ConnectionModifyTests(LDAPConnectionTests):
                    )
         connection = conn._getConnection()
         binduid, bindpwd = connection._last_bind[1]
-        self.assertEqual(binduid, u'')
+        self.assertTrue(isinstance(binduid, str))
+        self.assertEqual(binduid, '')
         self.assertEqual(bindpwd, '')
 
     def test_modify_authentication(self):
@@ -89,7 +88,7 @@ class ConnectionModifyTests(LDAPConnectionTests):
 
     def test_modify_explicit_delete(self):
         conn = self._makeSimple()
-        conn.insert('dc=localhost', 'cn=foo', attrs={'a': 'a', 'b':'b'})
+        conn.insert('dc=localhost', 'cn=foo', attrs={'a': 'a', 'b': ['b', 'c']})
         import ldap
         conn.modify( 'cn=foo,dc=localhost'
                    , mod_type=ldap.MOD_DELETE
@@ -98,6 +97,14 @@ class ConnectionModifyTests(LDAPConnectionTests):
         rec = conn.search('dc=localhost', fltr='(cn=foo)')['results'][0]
         self.failIf(rec.get('a'))
 
+        # Delete a subset of a multi-valued field
+        conn.modify( 'cn=foo,dc=localhost'
+                   , mod_type=ldap.MOD_DELETE
+                   , attrs={'b':['c']}
+                   )
+        rec = conn.search('dc=localhost', fltr='(cn=foo)')['results'][0]
+        self.assertEquals(rec['b'], ['b'])
+
         # Trying to modify the record by providing an empty or non-matching
         # value should not result in any changes.
         conn.modify( 'cn=foo,dc=localhost'
