@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2008-2010 Jens Vagelpohl and Contributors. All Rights Reserved.
+# Copyright (c) 2008-2012 Jens Vagelpohl and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
@@ -11,8 +11,6 @@
 #
 ##############################################################################
 """ test_connection_search: Tests for the LDAPConnection search method
-
-$Id: test_connection_search.py 1916 2010-04-12 11:50:39Z jens $
 """
 
 import unittest
@@ -28,7 +26,8 @@ class ConnectionSearchTests(LDAPConnectionTests):
         response = conn.search('dc=localhost', 'scope')
         connection = conn._getConnection()
         binduid, bindpwd = connection._last_bind[1]
-        self.assertEqual(binduid, u'')
+        self.assertTrue(isinstance(binduid, str))
+        self.assertEqual(binduid, '')
         self.assertEqual(bindpwd, '')
 
     def test_search_authentication(self):
